import os
import sys
import time

import typer
import subprocess
from rich import print

from ..tools.visual_call_graph import ProjectAnalyzer


app = typer.Typer(no_args_is_help=True)


@app.callback()
def callback():
    """
    Awesome AI CLI tool under development.
    """
    pass


@app.command(name="gpu")
def nvidia_info(
    refresh: bool = typer.Option(
        False, "--refresh", "-r", help="Enable real-time refresh"
    ),
    interval: float = typer.Option(
        2.0, "--interval", "-i", help="Refresh interval in seconds"
    ),
):
    """
    Check GPU driver information, PyTorch version, Python version, and Python executable path.

    Use --refresh or -r to enable real-time monitoring.
    Use --interval or -i to set the refresh interval (default: 2 seconds).
    """

    # 清屏函数，跨平台支持
    def clear_screen():
        os.system("cls" if os.name == "nt" else "clear")

    pytorch_info = ""
    try:
        import torch

        pytorch_info = f"PyTorch Version: {torch.__version__}\nCuda is available: {torch.cuda.is_available()}"
    except ImportError:
        pytorch_info = "[bold red]PyTorch is not installed.[/bold red]"

    # 获取 Python 版本和执行路径
    python_info = (
        f"Python Version: {sys.version}\nPython Executable Path: {sys.executable}"
    )

    # 显示 NVIDIA-SMI 信息的函数
    def show_gpu_info():
        result = subprocess.run(
            ["nvidia-smi"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,  # 返回文本（字符串）而不是字节
        )
        output = result.stdout
        error_output = result.stderr
        if result.returncode == 0:
            content = "\r\n".join(output.splitlines()[1:12])
            first_line = output.splitlines()[0]
            lenght = len(output.splitlines()[3])
            print("INFO".center(lenght, "="))
            print(f"Current Time: [green]{first_line}[/green]")
            print(content)
        else:
            print("NVIDIA-SMI Error Output:")
            print(error_output)

        # 打印 PyTorch 信息
        print("\n" + pytorch_info)
        # 打印 Python 信息
        print("\n" + python_info)

        if refresh:
            print(
                f"\n[italic cyan]Refreshing GPU info every {interval} seconds. Press Ctrl+C to exit.[/italic cyan]"
            )

    # 是否需要实时刷新
    if refresh:
        try:
            while True:
                clear_screen()
                show_gpu_info()
                time.sleep(interval)
        except KeyboardInterrupt:
            print("\n[bold green]GPU monitoring stopped.[/bold green]")
    else:
        show_gpu_info()


@app.command()
def test(name: str = typer.Option(None, "--name", "-n", help="this is a test param")):
    """
    this is test
    """
    print("It looks like it's correct.")


@app.command()
def callgraph(
    path: str = typer.Argument(
        ..., help="Path to the Python file or project directory to analyze."
    ),
    output: str = typer.Option(
        "call_graph.dot", "--output", "-o", help="Output path for the .dot file."
    ),
):
    """
    Generates a focused, intra-project function call graph.

    Analyzes a single Python file or an entire project directory and
    creates a .dot file representing the internal call structure.
    """
    target_path = os.path.abspath(path)
    print(f"🔍 Analyzing target: [bold cyan]{target_path}[/bold cyan]")

    project_root = None
    files_to_analyze = None

    if not os.path.exists(target_path):
        print(
            f"[bold red]Error:[/bold red] The provided path '{target_path}' does not exist."
        )
        raise typer.Exit(code=1)

    if os.path.isfile(target_path):
        # 如果是文件，项目根目录是其所在目录，分析列表只包含它自己
        print("Mode: Single File Analysis")
        project_root = os.path.dirname(target_path)
        files_to_analyze = [target_path]
    elif os.path.isdir(target_path):
        # 如果是目录，项目根目录就是它自己，分析列表由脚本自动发现
        print("Mode: Project Directory Analysis")
        project_root = target_path
        # files_to_analyze 保持为 None, 让 analyze 方法自己去发现
        files_to_analyze = None

    try:
        # 实例化并运行分析器
        analyzer = ProjectAnalyzer(project_root)
        analyzer.analyze(files_to_analyze=files_to_analyze)
        analyzer.generate_dot_file(output)  # 调用新的生成方法
        print(
            "\n[bold green]✨ Analysis complete! You can now render the DOT file with Graphviz:[/bold green]"
        )
        print(f"dot -Tpng {output} -o {os.path.splitext(output)[0]}.png")
    except Exception as e:
        print(f"[bold red]An unexpected error occurred:[/bold red] {e}")
        import traceback

        traceback.print_exc()  # 打印详细的错误堆栈，便于调试
        raise typer.Exit(code=1)


if __name__ == "__main__":
    app()



# tests/test_config.py

from ai4one.config import load_config, BaseConfig
from unittest.mock import patch
from dataclasses import field
from typing import List, Literal


# 从您的源码中导入被测试的类
def test_add():
    config = load_config("./pyproject.toml")
    assert config["project"]["name"] == "ai4one"


# --- 定义用于测试的配置类 ---


class SimpleConfig(BaseConfig):
    name: str
    value: int = 10
    test_list1: list
    test_list2: list = [1, 2, 3]
    test_list3: list[int] = [1, 2, 3]
    test_list4: List[int] = field(default_factory=lambda: [2025, 8, 1])
    test_list5: List[int] = [4, 5, 6]


class DataConfig(BaseConfig):
    path: str = "/data/default"
    batch_size: int = 32


class ModelConfig(BaseConfig):
    name: str = "default_model"
    layers: int = 4
    device: Literal["auto", "gpu", "cpu"] = "auto"


class NestedConfig(BaseConfig):
    data: DataConfig
    model: ModelConfig
    learning_rate: float = 0.01


# --- 测试类 ---


class TestBaseConfig:

    def test_inheritance_and_serialization(self):
        config = SimpleConfig(name="test_item")
        assert config.name == "test_item"
        assert config.value == 10
        assert config.test_list1 == []
        assert config.test_list3 == [1, 2, 3]
        assert config.test_list4 == [2025, 8, 1]
        assert config.test_list5 == [4, 5, 6]

        # 验证 to_json() 方法是否存在且工作正常
        # json_str = config.to_json()
        # assert '"name": "test_item"' in json_str
        # assert '"value": 10' in json_str

    def test_file_io_cycle(self, tmp_path):
        """测试 to_file 和 from_file 的完整读写循环。

        `tmp_path` 是一个 pytest fixture，提供一个临时的目录对象。
        """
        config_file = tmp_path / "config.json"
        original_config = SimpleConfig(name="cycle_test", value=99)

        original_config.to_file(config_file)
        assert config_file.exists()

        loaded_config = SimpleConfig.from_file(config_file)

        assert loaded_config == original_config

    def test_nested_config_io(self, tmp_path):
        """测试对嵌套配置的序列化和文件I/O。"""
        config_file = tmp_path / "nested_config.json"

        original_config = NestedConfig()
        original_config.data.batch_size = 128
        original_config.model.name = "CustomNet"

        original_config.to_file(config_file)

        loaded_config = NestedConfig.from_file(config_file)

        assert loaded_config.learning_rate == 0.01
        assert loaded_config.data.batch_size == 128
        assert loaded_config.model.name == "CustomNet"

        assert isinstance(loaded_config.data, DataConfig)
        assert isinstance(loaded_config.model, ModelConfig)

        assert loaded_config == original_config

    def test_argument_parser_override(self):
        """测试 argument_parser 是否能正确地被命令行参数覆盖。"""
        # 模拟命令行输入: python script.py --learning_rate 0.5 --path /new/path
        cli_args = [
            "script.py",  # 第一个参数总是脚本名
            "--learning_rate",
            "0.5",
            "--path",
            "/new/path",
        ]

        with patch("sys.argv", cli_args):
            parsed_config = NestedConfig.argument_parser()

        assert parsed_config.model.name == "default_model"

        assert parsed_config.learning_rate == 0.5
        assert parsed_config.data.path == "/new/path"



from dataclasses import field, dataclass  # noqa
from typing import Type, TypeVar, List  # noqa
from pathlib import Path


from dataclasses_json import dataclass_json
from simple_parsing import ArgumentParser


T = TypeVar("T", bound="BaseConfig")


def load_config(
    path: str,
):
    import tomllib  # python3.11 引入的库，低版本没有
    with open(path, mode="rb") as f:
        return tomllib.load(f)


class BaseConfig:
    """配置基类，可以实现自动解析命令行参数 argument_parser, rom_file/to_file, from_json/to_json

    一个支持 JSON, YAML, TOML 的多功能配置基类。
    ---
    ### 使用示例 1: 基础用法与文件操作

    定义一个简单的配置类，设置其字段和默认值。

    >>> class DatabaseConfig(BaseConfig):
    ...     host: str = "localhost"
    ...     port: int = 5432
    ...     user: str

    保存和加载配置。

    >>> db_conf = DatabaseConfig(user="admin")
    >>> db_conf.to_file("db.json")
    >>> loaded_conf = DatabaseConfig.from_file("db.json")
    >>> print(loaded_conf.user)
    admin

    ---
    ### 使用示例 2: 嵌套配置 (Hierarchical Configuration)

    这是管理复杂项目配置的推荐方式。首先定义各个子配置。

    >>> class DataConfig(BaseConfig):
    ...     path: str = "/data/set"
    ...     batch_size: int = 64
    >>>
    >>> class ModelConfig(BaseConfig):
    ...     name: str = "ResNet50"
    ...     embedding_dim: int = 256

    然后，将它们组合成一个主配置类。使用 `dataclasses.field` 和 `default_factory`
    来确保每个子配置都能被正确初始化。

    >>> from dataclasses import field
    >>>
    >>> class MainConfig(BaseConfig):
    ...     data: DataConfig = field(default_factory=DataConfig)
    ...     model: ModelConfig = field(default_factory=ModelConfig)
    ...     learning_rate: float = 0.001

    主配置可以作为一个整体进行序列化和反序列化。

    >>> main_conf = MainConfig()
    >>> print(main_conf.model.name)
    ResNet50
    >>> main_conf.to_file("config.json")

    ---
    ### 使用示例 3: 命令行参数解析

    `argument_parser()` 方法允许你用命令行参数覆盖文件或代码中的默认配置。

    >>> # 在你的 Python 脚本 (例如: train.py) 中:
    >>> # config = MainConfig.argument_parser()
    >>> # print(config.learning_rate)
    >>> # print(config.data.batch_size)

    然后你可以在终端中运行脚本并传入参数：

    .. code-block:: shell

        # 使用默认值运行
        # python train.py
        # 输出: 0.001
        # 输出: 64

        # 从命令行覆盖参数
        # python train.py --learning_rate 0.1 --batch_size 128
        # 输出: 0.1
        # 输出: 128
    """

    def __init_subclass__(cls: Type, **kwargs):
        super().__init_subclass__(**kwargs)

        dataclass(cls)
        dataclass_json(cls)

    def to_file(self, file_path: str, **kwargs):
        file_path_obj = Path(file_path)
        file_path_obj.parent.mkdir(parents=True, exist_ok=True)

        content = self.to_dict()  # 使用 dataclasses-json 的方法转换为字典

        ext = file_path_obj.suffix
        with open(file_path_obj, "w", encoding="utf-8") as f:
            if ext == ".json":
                import json

                json.dump(content, f, indent=2, ensure_ascii=False, **kwargs)
            elif ext in (".yaml", ".yml"):
                try:
                    import yaml
                except ImportError:
                    raise ImportError(
                        "PyYAML is not installed. Please run 'pip install pyyaml'"
                    )
                yaml.dump(content, f, **kwargs)
            elif ext == ".toml":
                try:
                    import toml
                except ImportError:
                    raise ImportError(
                        "toml is not installed. Please run 'pip install toml'"
                    )
                toml.dump(content, f, **kwargs)
            else:
                raise ValueError(
                    f"Unsupported file format: {ext}. Please use .json, .yaml, or .toml"
                )

    @classmethod
    def from_file(cls: Type[T], file_path: str) -> T:
        """
        从文件加载配置实例，根据文件扩展名自动选择格式。

        Args:
            file_path (str): 源文件路径 (e.g., 'config.json', 'config.yaml').

        Returns:
            一个填充了文件数据的配置类实例。
        """
        file_path_obj = Path(file_path)
        if not file_path_obj.exists():
            raise FileNotFoundError(f"Configuration file not found: {file_path}")

        ext = file_path_obj.suffix
        with open(file_path_obj, "r", encoding="utf-8") as f:
            if ext == ".json":
                import json

                content = json.load(f)
            elif ext in (".yaml", ".yml"):
                try:
                    import yaml
                except ImportError:
                    raise ImportError(
                        "PyYAML is not installed. Please run 'pip install pyyaml'"
                    )
                content = yaml.safe_load(f)
            elif ext == ".toml":
                try:
                    import toml
                except ImportError:
                    raise ImportError(
                        "toml is not installed. Please run 'pip install toml'"
                    )
                content = toml.load(f)
            else:
                raise ValueError(
                    f"Unsupported file format: {ext}. Please use .json, .yaml, or .toml"
                )

        return cls.from_dict(content)

    @classmethod
    def argument_parser(cls: Type[T]) -> T:
        parser = ArgumentParser()
        parser.add_arguments(cls, dest="config")
        args = parser.parse_args()
        return args.config



import ast
import os
import argparse
from collections import defaultdict
from typing import Dict, List, Optional


class DefinitionVisitor(ast.NodeVisitor):
    def __init__(self, module_name: str):
        self.module_name = module_name
        self.definitions = {"functions": set(), "classes": defaultdict(set)}

    def visit_FunctionDef(self, node: ast.FunctionDef):
        self.definitions["functions"].add(f"{self.module_name}.{node.name}")

    def visit_ClassDef(self, node: ast.ClassDef):
        class_name = f"{self.module_name}.{node.name}"
        self.definitions["classes"][class_name] = set()
        for item in node.body:
            if isinstance(item, (ast.FunctionDef, ast.AsyncFunctionDef)):
                method_name = f"{class_name}.{item.name}"
                self.definitions["classes"][class_name].add(method_name)


class CallVisitor(ast.NodeVisitor):
    def __init__(
        self,
        module_name: str,
        imports: Dict,
        from_imports: Dict,
        analyzer: "ProjectAnalyzer",
    ):
        self.module_name = module_name
        self.imports = imports
        self.from_imports = from_imports
        self.analyzer = analyzer
        self.graph = defaultdict(set)
        self.scope_stack: List[Dict[str, str]] = [{}]
        self.current_path: List[str] = []

    def _get_current_fqn(self) -> Optional[str]:
        if not self.current_path:
            return self.module_name
        return f"{self.module_name}.{'.'.join(self.current_path)}"

    def visit_FunctionDef(self, node: ast.FunctionDef):
        self.current_path.append(node.name)
        self.scope_stack.append({})
        for arg in node.args.args:
            if arg.arg != "self":
                self.scope_stack[-1][arg.arg] = "typing.Any"
        self.generic_visit(node)
        self.scope_stack.pop()
        self.current_path.pop()

    visit_AsyncFunctionDef = visit_FunctionDef

    def visit_ClassDef(self, node: ast.ClassDef):
        self.current_path.append(node.name)
        self.generic_visit(node)
        self.current_path.pop()

    def visit_Assign(self, node: ast.Assign):
        if len(node.targets) == 1 and isinstance(node.targets[0], ast.Name):
            target_name = node.targets[0].id
            if isinstance(node.value, ast.Call):
                callee_fqn = self._resolve_call_fqn(node.value.func)
                if callee_fqn and self.analyzer._is_class(callee_fqn):
                    self.scope_stack[-1][target_name] = callee_fqn
        self.generic_visit(node)

    def visit_Call(self, node: ast.Call):
        caller_fqn = self._get_current_fqn()
        callee_fqn_candidate = self._resolve_call_fqn(node.func)
        if caller_fqn and callee_fqn_candidate:
            final_callee = self.analyzer._filter_and_resolve_callee(
                callee_fqn_candidate
            )
            if final_callee:
                self.graph[caller_fqn].add(final_callee)
        self.generic_visit(node)

    def _resolve_call_fqn(self, node: ast.AST) -> Optional[str]:
        if isinstance(node, ast.Name):
            name = node.id
            for scope in reversed(self.scope_stack):
                if name in scope:
                    return scope[name]
            if name in self.from_imports:
                return f"{self.from_imports[name]}.{name}"
            if name in self.imports:
                return self.imports[name]
            return f"{self.module_name}.{name}"
        elif isinstance(node, ast.Attribute):
            prefix_type_fqn = self._resolve_call_fqn(node.value)
            if isinstance(node.value, ast.Name) and node.value.id == "self":
                prefix_type_fqn = (
                    f"{self.module_name}.{'.'.join(self.current_path[:-1])}"
                )
            return f"{prefix_type_fqn}.{node.attr}" if prefix_type_fqn else None
        return None


class ProjectAnalyzer:
    def __init__(self, project_root: str):
        self.project_root = os.path.abspath(project_root)
        self.call_graph = defaultdict(set)
        self.file_info: Dict[str, Dict] = {}
        self.project_definitions: Dict[str, Dict] = {}

    def analyze(self, files_to_analyze: Optional[List[str]] = None):
        """
        分析项目。如果提供了files_to_analyze列表，则只分析这些文件。
        否则，自动发现项目根目录下的所有文件。
        """
        if files_to_analyze:
            py_files_to_scan = (
                self._discover_files()
            )  # 仍然扫描所有文件以建立完整的定义
            py_files_to_analyze = files_to_analyze
        else:
            py_files_to_scan = self._discover_files()
            py_files_to_analyze = py_files_to_scan

        for filepath in py_files_to_scan:
            self._gather_definitions(filepath)
        for filepath in py_files_to_scan:
            self._pre_analyze_imports(filepath)
        for filepath in py_files_to_analyze:  # 只对指定的文件进行调用分析
            self._analyze_file_calls(filepath)

    def _is_class(self, fqn: str) -> bool:
        module_name = ".".join(fqn.split(".")[:-1])
        return (
            module_name in self.project_definitions
            and fqn in self.project_definitions[module_name].get("classes", {})
        )

    def _is_method(self, fqn: str) -> bool:
        parts = fqn.split(".")
        if len(parts) < 2:
            return False
        class_fqn = ".".join(parts[:-1])
        class_module_name = ".".join(class_fqn.split(".")[:-1])
        if (
            class_module_name in self.project_definitions
            and class_fqn
            in self.project_definitions[class_module_name].get("classes", {})
        ):
            return (
                fqn in self.project_definitions[class_module_name]["classes"][class_fqn]
            )
        return False

    def _filter_and_resolve_callee(self, fqn: str) -> Optional[str]:
        parts = fqn.split(".")
        module_name = ".".join(parts[:-1])
        if module_name in self.project_definitions and fqn in self.project_definitions[
            module_name
        ].get("functions", set()):
            return fqn
        if self._is_method(fqn):
            return fqn
        if self._is_class(fqn):
            init_method_fqn = f"{fqn}.__init__"
            if self._is_method(init_method_fqn):
                return init_method_fqn
        return None

    def _discover_files(self) -> List[str]:
        py_files = []
        for root, _, files in os.walk(self.project_root):
            for file in files:
                if file.endswith(".py"):
                    py_files.append(os.path.join(root, file))
        return py_files

    def _filepath_to_module(self, filepath: str) -> str:
        rel_path = os.path.relpath(filepath, self.project_root)
        if os.path.basename(rel_path) == "__init__.py":
            rel_path = os.path.dirname(rel_path) or ""
        module_path = os.path.splitext(rel_path)[0]
        return module_path.replace(os.path.sep, ".")

    def _gather_definitions(self, filepath: str):
        module_name = self._filepath_to_module(filepath)
        self.project_definitions[module_name] = {
            "functions": set(),
            "classes": defaultdict(set),
        }
        with open(filepath, "r", encoding="utf-8") as f:
            try:
                tree = ast.parse(f.read(), filename=filepath)
                visitor = DefinitionVisitor(module_name)
                visitor.visit(tree)
                self.project_definitions[module_name] = visitor.definitions
            except SyntaxError as e:
                print(f"Skipping file with syntax error: {filepath}\n{e}")

    def _pre_analyze_imports(self, filepath: str):
        imports: Dict[str, str] = {}
        from_imports: Dict[str, str] = {}
        with open(filepath, "r", encoding="utf-8") as f:
            try:
                tree = ast.parse(f.read(), filename=filepath)
            except SyntaxError:
                return
        for node in ast.walk(tree):
            if isinstance(node, ast.Import):
                for alias in node.names:
                    imports[alias.asname or alias.name] = alias.name
            elif isinstance(node, ast.ImportFrom):
                if node.level == 0 and node.module:
                    for alias in node.names:
                        from_imports[alias.asname or alias.name] = node.module
        self.file_info[filepath] = {"imports": imports, "from_imports": from_imports}

    def _analyze_file_calls(self, filepath: str):
        if filepath not in self.file_info:
            return
        module_name = self._filepath_to_module(filepath)
        info = self.file_info[filepath]

        with open(filepath, "r", encoding="utf-8") as f:
            try:
                tree = ast.parse(f.read(), filename=filepath)
            except SyntaxError:
                # 如果文件有语法错误，则跳过
                return

        # 传递 analyzer 实例 (self) 给 visitor
        visitor = CallVisitor(module_name, info["imports"], info["from_imports"], self)
        visitor.visit(tree)  # 现在 'tree' 变量已经被正确定义了

        for caller, callees in visitor.graph.items():
            self.call_graph[caller].update(callees)

    def generate_dot_file(self, output_path: str):
        # (This part is unchanged, so I'll omit it for brevity, but it should be copied from the previous version)
        lines = [
            "digraph CallGraph {",
            "    rankdir=LR;",
            '    node [shape=box, style=rounded, fontname="Arial"];',
            '    edge [fontname="Arial", fontsize=10];',
            "    graph [splines=ortho];\n",
        ]
        all_nodes = set(self.call_graph.keys())
        for callees in self.call_graph.values():
            all_nodes.update(callees)
        module_contents = defaultdict(
            lambda: {"classes": defaultdict(list), "functions": []}
        )
        for fqn in all_nodes:
            parts = fqn.split(".")
            is_method = self._is_method(fqn)
            if is_method:
                module_name = ".".join(parts[:-2])
                class_name = ".".join(parts[:-1])
                module_contents[module_name]["classes"][class_name].append(fqn)
            else:
                module_name = ".".join(parts[:-1]) if "." in fqn else fqn
                module_contents[module_name]["functions"].append(fqn)
        for i, (module_name, contents) in enumerate(module_contents.items()):
            if not module_name:
                continue
            lines.append(f"    subgraph cluster_module_{i} {{")
            lines.append(f'        label = "Module: {module_name}";')
            lines.append('        style=filled; color="#EFEFEF";')
            for func_fqn in contents["functions"]:
                func_name = func_fqn.split(".")[-1]
                lines.append(f'        "{func_fqn}" [label="{func_name}"];')
            for j, (class_name, methods) in enumerate(contents["classes"].items()):
                lines.append(f"        subgraph cluster_class_{i}_{j} {{")
                lines.append(
                    f"            label = \"Class: {class_name.split('.')[-1]}\";"
                )
                lines.append('            style=filled; color="#DDEEFF";')
                lines.append("            node [style=filled, fillcolor=white];")
                for method_fqn in methods:
                    method_short_name = ".".join(method_fqn.split(".")[-2:])
                    lines.append(
                        f'            "{method_fqn}" [label="{method_short_name}"];'
                    )
                lines.append("        }")
            lines.append("    }\n")
        for caller, callees in self.call_graph.items():
            if not callees:
                continue
            for callee in callees:
                lines.append(f'    "{caller}" -> "{callee}";')
        lines.append("}")
        dot_content = "\n".join(lines)
        with open(output_path, "w", encoding="utf-8") as f:
            f.write(dot_content)
        print(f"✅ DOT source file generated: {output_path}")
